﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_subtlety.lua
--
--	Copyright (C) 2012 Deronda
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  Some of the code was pulled
--  from existing clcInfo modules and the example module.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Deronda (@ Stormreaver)
------------------------------------------------------------------------------
-- Description:
--   Actions module for subtlety rogues
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Rogue then return end -- need clcInfo_Rogue
local _, englishClass = UnitClass("player") -- need to be a rogue
if englishClass ~= clcInfo_Rogue.CLASS then return end

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Rogue = clcInfo_Rogue
local spells = clcInfo_Rogue.spells
local GetTimeLeft = clcInfo_Rogue.GetTimeLeft
local GetCooldown = clcInfo_Rogue.GetCooldown
local IconNext = clcInfo_Rogue.IconNext
local IconCurrent = clcInfo_Rogue.IconCurrent
local BleedDebuffLeftOnTarget = clcInfo_Rogue.BleedDebuffLeftOnTarget
local IsStealthed = clcInfo_Rogue.IsStealthed
local UnitPower = UnitPower
local UnitAura = UnitAura
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers

-- Assign module variables
local specname = "subtlety"
local prio_arg = "subtletyprio"
local sample_prio = "tricks sd vanish ss premed ambush prep snd rupture recup eviscerate hemo backstab"
local prio = "sd ambush snd rupture recup eviscerate hemo backstab"
local class_module
local class_module_name
local has_hemo_glyph

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Rogue.VERSION,
    prio = prio,
    ignore_rupture_no_bleed = false,
    rangePerSkill = true,
}
local tdb

-- actions -------------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action functions up
-- we want the action functions to run as quickly as possible
local Ambush = spells["Ambush"].name
local FindWeakness = spells["Find Weakness"].name
local MasterOfSubtlety = spells["Master of Subtlety"].name
local Premeditation = spells["Premeditation"].name
local Recuperate = spells["Recuperate"].name
local Rupture = spells["Rupture"].name
local ShadowDance = spells["Shadow Dance"].name
local SliceAndDice = spells["Slice and Dice"].name
local Stealth = spells["Stealth"].name
local Vanish = spells["Vanish"].name
local FuryoftheDestroyer = spells["Fury of the Destroyer"].name

local AmbushId = spells["Ambush"].id
local BackstabId = spells["Backstab"].id
local EviscerateId = spells["Eviscerate"].id
local Hemorrhageid = spells["Hemorrhage"].id
local PremeditationId = spells["Premeditation"].id
local PreparationId = spells["Preparation"].id
local RecuperateId = spells["Recuperate"].id
local RuptureId = spells["Rupture"].id
local ShadowDanceId = spells["Shadow Dance"].id
local ShadowstepId = spells["Shadowstep"].id
local SliceAndDiceId = spells["Slice and Dice"].id
local TricksId = spells["Tricks of the Trade"].id
local VanishId = spells["Vanish"].id

local actions  = {
	tricks = {
		id = TricksId,
		GetCD = function(cur_id)
            if cur_id ~= TricksId and (GetNumRaidMembers() > 0 or GetNumPartyMembers() > 0) then
                return GetCooldown(TricksId)
            end
            return 100
        end,
		UpdateStatus = function(combo_points)
            return combo_points
		end,
		info = spells["Tricks of the Trade"].name,
	},
	sd = {
		id = ShadowDanceId,
		GetCD = function(_, combo_points)
	        if not IsStealthed() and combo_points < 5
            then
                local yellow = (UnitPower("player") < 86) --pool energy
                return GetCooldown(ShadowDanceId), yellow
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points, true -- true = ignore time increment since not on gcd
		end,
		info = spells["Shadow Dance"].name,
	},
	vanish = {
		id = VanishId,
		GetCD = function(cur_id, combo_points)
            if combo_points < 2
               and not IsStealthed()
               and GetCooldown(ShadowstepId) == 0
               and GetTimeLeft("player", MasterOfSubtlety, "HELPFUL") == 0
               and GetTimeLeft("player", FindWeakness, "HELPFUL") == 0
            then
                local yellow = (UnitPower("player") < 61) --pool energy
                local vanishcd = ((cur_id == PreparationId) and 0) or GetCooldown(VanishId)
                return vanishcd, yellow
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
			return combo_points, true -- true = ignore time increment since not on gcd
		end,
		info = spells["Vanish"].name,
	},
	ss = {
		id = ShadowstepId,
		GetCD = function(cur_id)
	        if cur_id ~= ShadowstepId and IsStealthed() then
                return ((cur_id == PreparationId) and 0) or GetCooldown(ShadowstepId)
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points, true -- true = ignore time increment since not on gcd
		end,
		info = spells["Shadowstep"].name,
	},
    premed = {
		id = PremeditationId,
		GetCD = function(cur_id, combo_points)
	        if cur_id ~= PremeditationId
               and combo_points < 3
               and IsStealthed()
            then
                return GetCooldown(PremeditationId)
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points + 2, true -- true = ignore time increment since not on gcd
		end,
		info = spells["Premeditation"].name,
	},
    ambush = {
		id = AmbushId,
		GetCD = function(cur_id, combo_points)
	        if combo_points < 5
               and IsStealthed()
               and not (cur_id == AmbushId and GetTimeLeft("player", ShadowDance, "HELPFUL") == 0)
            then
                return GetCooldown(AmbushId)
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points + 2
		end,
		info = spells["Ambush"].name,
	},
    prep = {
		id = PreparationId,
		GetCD = function(cur_id)
	        if cur_id ~= PreparationId and GetCooldown(VanishId) > 60 then
                return GetCooldown(PreparationId)
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points
		end,
		info = spells["Preparation"].name,
	},
	snd = {
		id = SliceAndDiceId,
		GetCD = function(cur_id, combo_points)
	        if cur_id ~= SliceAndDiceId 
               and combo_points == 5
               and GetTimeLeft("player", SliceAndDice, "HELPFUL") < 3
            then
                return GetCooldown(SliceAndDiceId)
            end
            return 100
		end,
		UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
		end,
		info = spells["Slice and Dice"].name,
	},
    rupture = {
		id = RuptureId,
		GetCD = function(cur_id, combo_points)
            if cur_id ~= RuptureId
               and combo_points == 5 
               and GetTimeLeft("target", Rupture, "HARMFUL|PLAYER") == 0
               and (not tdb.ignore_rupture_no_bleed or BleedDebuffLeftOnTarget() > 0)
            then
               return GetCooldown(RuptureId)
            end
            return 100
		end,
		UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
		end,
		info = spells["Rupture"].name,
	},
    recup = {
		id = RecuperateId,
		GetCD = function(cur_id, combo_points)
            if cur_id ~= RecuperateId
               and combo_points == 5 
               and GetTimeLeft("player", Recuperate, "HELPFUL") < 3
               and GetTimeLeft("target", Rupture, "HARMFUL|PLAYER") > 6
            then
               return GetCooldown(RecuperateId)
            end
            return 100
		end,
		UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
		end,
		info = spells["Recuperate"].name,
	},
    eviscerate = {
		id = EviscerateId,
		GetCD = function(_, combo_points)
            if combo_points == 5
               and GetTimeLeft("target", Rupture, "HARMFUL|PLAYER") > 0
            then
                return GetCooldown(EviscerateId)
            end
            return 100
		end,
		UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
		end,
		info = spells["Eviscerate"].name,
	},
    hemo = {
		id = Hemorrhageid,
		GetCD = function(cur_id, combo_points)
            -- 89775 == Glyphed Hemo bleed damage. UnitAura(unit, "Hemorrhage") only returns
            --          info on the bleed debuff, not the bleed dot from the glyph.  To find
            --          the time left on the dot you have to iterate through all the debuffs
            --          until you find the one with spell ID 89775.  GetTimeLeft() does this,
            --          but it runs in O(N) time when using a spell ID.  So to speed things up
            --          for sub rogues that aren't glyphed hemo, we keep track of whether or not
            --          hemo is glyphed.  If it isn't, GetTimeLeft is never called on 89775.
            if cur_id ~= Hemorrhageid and not IsStealthed() then
                if has_hemo_glyph then
                    if GetTimeLeft("target", 89775, "HARMFUL|PLAYER") < 4 then
                        if combo_points < 4 then
                            local yellow = UnitPower("player") < 41
                            return GetCooldown(Hemorrhageid), yellow
                        elseif combo_points < 5 then
                            local yellow = UnitPower("player") < 81
                            return GetCooldown(Hemorrhageid), yellow
                        end
                    end
                elseif BleedDebuffLeftOnTarget() < 2 and combo_points < 5 then
                    return GetCooldown(Hemorrhageid)
                end
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points + 1
		end,
		info = spells["Hemorrhage"].name,
	},  
    backstab = {
		id = BackstabId,
		GetCD = function(_, combo_points)
            if not IsStealthed() then
                if combo_points < 4 then
                    local yellow = UnitPower("player") < 41
                    return GetCooldown(BackstabId), yellow
                elseif combo_points < 5 then
                    local yellow = UnitPower("player") < 81
                    return GetCooldown(BackstabId), yellow
                end
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points + 1
		end,
		info = spells["Backstab"].name,
	},  
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB

--initialize module.
class_module, class_module_name = clcInfo_Rogue:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
	tdb, GetTDB, SetTDB = clcInfo_Rogue:OnTemplatesUpdate(class_module_name, actions, tdb_default, specname)
    --if the module switches to sub, check for the hemo glyph
    has_hemo_glyph = clcInfo_Rogue.HasGlyphSocketed(56807)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
	db, GetDB, SetDB = clcInfo_Rogue:OnInitialize(class_module_name, db_default)
end

-- called when module options are opened
local function LoadSubtletyModule()
    local _, talent_tree_name = GetTalentTabInfo(3)
    clcInfo_Rogue.LoadModule(class_module_name, talent_tree_name, prio_arg, sample_prio,
                             actions, GetDB, SetDB, GetTDB, SetTDB)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadSubtletyModule

-- giving queue as command line
local function CmdSubtlety(args)
	clcInfo_Rogue:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdSubtlety

-- events
-- if the user changes her glyphs, check to see if Hemo is there.
function clcInfo_Rogue.GLYPH_REMOVED()
    has_hemo_glyph = clcInfo_Rogue.HasGlyphSocketed(56807)
end

function clcInfo_Rogue.GLYPH_ADDED()
    has_hemo_glyph = clcInfo_Rogue.HasGlyphSocketed(56807)
end

clcInfo_Rogue.EventFrame:RegisterEvent("GLYPH_ADDED")
clcInfo_Rogue.EventFrame:RegisterEvent("GLYPH_REMOVED")

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconSubtletyCurrent(...)
	return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconSubtletyNext(...)
	return IconNext()
end
